@@ -1,5 +1,5 @@
 /****************************************************************************************
- * Copyright (c) 2010 Leo Franchi <lfranchi@kde.org>                                    *
+ * Copyright (c) 2010-2012 Leo Franchi <lfranchi@kde.org>                               *
  *                                                                                      *
  * This program is free software; you can redistribute it and/or modify it under        *
  * the terms of the GNU General Public License as published by the Free Software        *
@@ -94,6 +94,7 @@ namespace Parser
     License parseLicense( QXmlStreamReader& xml ) throw( ParseError );
 
     QByteArray parsePlaylistSessionId( QXmlStreamReader& xml ) throw( ParseError );
+    SongList parseDynamicLookahead( QXmlStreamReader& xml ) throw( ParseError );
 
     // Catalog functions
     Catalogs parseCatalogList( QXmlStreamReader& xml ) throw( ParseError );
@@ -112,7 +113,7 @@ namespace Parser
 
     SessionInfo parseSessionInfo( QXmlStreamReader& xml ) throw( ParseError );
     QVector< QString > parseRulesList( QXmlStreamReader& xml ) throw( ParseError );
-    QVector< SessionItem > parseSessionSongItem( QXmlStreamReader& xml, const QString& type ) throw( ParseError );
+//     QVector< SessionItem > parseSessionSongItem( QXmlStreamReader& xml, const QString& type ) throw( ParseError );
 
 }
 }
